Public Class Form1
    ' Remember where we got it.
    Private m_DragSource As ListBox = Nothing

    ' Allow drag events.
    Private Sub Form1_Load(ByVal sender As System.Object, _
     ByVal e As System.EventArgs) Handles MyBase.Load
        lstUnselected.AllowDrop = True
        lstSelected.AllowDrop = True

        ' Add event handlers.
        AddHandler lstUnselected.MouseDown, AddressOf List_MouseDown
        AddHandler lstUnselected.DragOver, AddressOf List_DragOver
        AddHandler lstUnselected.DragDrop, AddressOf List_DragDrop
        AddHandler lstUnselected.DragLeave, AddressOf List_DragLeave

        AddHandler lstSelected.MouseDown, AddressOf List_MouseDown
        AddHandler lstSelected.DragOver, AddressOf List_DragOver
        AddHandler lstSelected.DragDrop, AddressOf List_DragDrop
        AddHandler lstSelected.DragLeave, AddressOf List_DragLeave
    End Sub

    ' Start a drag.
    Private Sub List_MouseDown(ByVal sender As Object, _
     ByVal e As System.Windows.Forms.MouseEventArgs)
        ' Make sure this is the right button.
        If e.Button <> Windows.Forms.MouseButtons.Right Then Exit Sub

        ' Select the item at this point.
        Dim this_list As ListBox = DirectCast(sender, ListBox)
        this_list.SelectedIndex = this_list.IndexFromPoint(e.X, e.Y)
        If this_list.SelectedIndex < 0 Then Exit Sub

        ' Remember where the drag started.
        m_DragSource = this_list

        ' Start the drag.
        this_list.DoDragDrop( _
            this_list.SelectedItem.ToString, _
            DragDropEffects.Move Or DragDropEffects.Copy)

        ' We're done dragging.
        m_DragSource = Nothing
    End Sub

    ' Highlight the item under the mouse.
    Private Sub List_DragOver(ByVal sender As Object, _
     ByVal e As System.Windows.Forms.DragEventArgs)
        If m_DragSource Is Nothing Then Exit Sub

        Dim this_list As ListBox = DirectCast(sender, ListBox)
        Dim pt As Point = _
            this_list.PointToClient(New Point(e.X, e.Y))
        Dim drop_index As Integer = _
            this_list.IndexFromPoint(pt.X, pt.Y)
        this_list.SelectedIndex = drop_index

        ' Display the Move or Copy cursor.
        Const KEY_CTRL As Integer = 8
        If (e.KeyState And KEY_CTRL) <> 0 Then
            e.Effect = DragDropEffects.Copy
        Else
            e.Effect = DragDropEffects.Move
        End If
    End Sub

    ' Unhighlight the target item when the drag leaves.
    Private Sub List_DragLeave(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim this_list As ListBox = DirectCast(sender, ListBox)
        this_list.SelectedIndex = -1
    End Sub

    ' Accept the drop.
    Private Sub List_DragDrop(ByVal sender As Object, _
     ByVal e As System.Windows.Forms.DragEventArgs)
        Dim this_list As ListBox = DirectCast(sender, ListBox)
        MoveItem(e.Data.GetData(DataFormats.Text).ToString, _
            m_DragSource, this_list, e.X, e.Y, _
            e.Effect = DragDropEffects.Move)
    End Sub

    ' Move the value txt from drag_source to drop_target.
    Private Sub MoveItem(ByVal txt As String, ByVal drag_source As ListBox, _
     ByVal drop_target As ListBox, ByVal X As Integer, ByVal Y As Integer, _
     ByVal move_item As Boolean)
        ' See which item is selected in the drop target.
        Dim drop_index As Integer = drop_target.SelectedIndex
        If drop_index < 0 Then
            ' Add at the end.
            drop_index = drop_target.Items.Add(txt)
        Else
            ' Add before the selected item.
            drop_target.Items.Insert(drop_target.SelectedIndex, txt)
        End If

        ' Select the item.
        drop_target.SelectedIndex = drop_index

        ' See if we are moving or copying.
        If move_item Then
            ' Remove the value from drag_source.
            If drag_source Is drop_target Then
                ' Make sure we don't remove the item we just added.
                Dim target_index As Integer = drag_source.FindStringExact(txt)
                If target_index = drop_index Then _
                    target_index = drag_source.FindStringExact(txt, target_index)
                drag_source.Items.RemoveAt(target_index)
            Else
                ' Remove the item.
                drag_source.Items.Remove(txt)
            End If
        End If
    End Sub
End Class
