' Add references to:
' System.Workflow.Runtime
' System.Workflow.Activities
' System.Workflow.ComponentModel

Imports System.Workflow.Runtime
Imports System.Workflow.Activities
Imports System.Workflow.Activities.Rules
Imports System.Workflow.ComponentModel

<ExternalDataExchange()> _
Public Interface IBugReportService
    Event EngineerAssigned As EventHandler(Of StringArgs)
    Event BugWorked As EventHandler(Of ExternalDataEventArgs)
    Event BugRejected As EventHandler(Of ExternalDataEventArgs)
    Event TestPassed As EventHandler(Of ExternalDataEventArgs)
    Event TestFailed As EventHandler(Of ExternalDataEventArgs)

    Sub AssignEngineer(ByVal bug_description As String)
    Sub WorkBug(ByVal bug_description As String)
    Sub TestBug(ByVal bug_description As String)
End Interface

Public Class BugReportWF
    Inherits SequentialWorkflowActivity

#Region "Properties"
    Private m_BugDescription As String = ""
    Public Property BugDescription() As String
        Get
            Return m_BugDescription
        End Get
        Set(ByVal value As String)
            m_BugDescription = value
        End Set
    End Property

    Private m_AssignedEngineer As String = ""
    Public Property AssignedEngineer() As String
        Get
            Return m_AssignedEngineer
        End Get
        Set(ByVal value As String)
            m_AssignedEngineer = value
        End Set
    End Property

    Private m_Status As String = ""
    Public Property Status() As String
        Get
            Return m_Status
        End Get
        Set(ByVal value As String)
            m_Status = value
        End Set
    End Property
#End Region ' Properties

    Private WithEvents m_AssignEngineer As CallExternalMethodActivity
    Private WithEvents m_EngineerAssignedEvent As HandleExternalEventActivity
    Private WithEvents m_WorkBug As CallExternalMethodActivity
    Private WithEvents m_BugWorkedEvent As HandleExternalEventActivity
    Private WithEvents m_BugRejectedEvent As HandleExternalEventActivity
    Private WithEvents m_TestBug As CallExternalMethodActivity
    Private WithEvents m_TestPassedEvent As HandleExternalEventActivity
    Private WithEvents m_TestFailedEvent As HandleExternalEventActivity

    Public Sub New()
        Me.CanModifyActivities = True

        ' Assign an engineer.
        m_AssignEngineer = _
            New CallExternalMethodActivity("invoke_AssignEngineer")
        m_AssignEngineer.InterfaceType = GetType(IBugReportService)
        m_AssignEngineer.MethodName = "AssignEngineer"

        Dim assign_param As New WorkflowParameterBinding
        assign_param.ParameterName = "bug_description"
        assign_param.Value = ""    ' We will change this later.
        m_AssignEngineer.ParameterBindings.Add(assign_param)

        ' Catch the EngineerAssigned event.
        m_EngineerAssignedEvent = New HandleExternalEventActivity("catch_EngineerAssigned")
        m_EngineerAssignedEvent.InterfaceType = GetType(IBugReportService)
        m_EngineerAssignedEvent.EventName = "EngineerAssigned"

        ' Work the bug.
        m_WorkBug = New CallExternalMethodActivity("invoke_WorkBug")
        m_WorkBug.InterfaceType = GetType(IBugReportService)
        m_WorkBug.MethodName = "WorkBug"

        Dim work_param As New WorkflowParameterBinding
        work_param.ParameterName = "bug_description"
        work_param.Value = ""    ' We will change this later.
        m_WorkBug.ParameterBindings.Add(work_param)

        ' Listen for either the BugWorked or BugRejected event.
        ' Catch the BugWorked event.
        m_BugWorkedEvent = New HandleExternalEventActivity("catch_BugWorked")
        m_BugWorkedEvent.InterfaceType = GetType(IBugReportService)
        m_BugWorkedEvent.EventName = "BugWorked"

        ' Catch the BugRejected event.
        m_BugRejectedEvent = New HandleExternalEventActivity("catch_BugRejected")
        m_BugRejectedEvent.InterfaceType = GetType(IBugReportService)
        m_BugRejectedEvent.EventName = "BugRejected"

        ' Make activities driven by the BugWorked and BugRejected events.
        Dim eventdriven_worked As New EventDrivenActivity("eventdriven_Worked")
        eventdriven_worked.Activities.Add(m_BugWorkedEvent)
        Dim eventdriven_rejected As New EventDrivenActivity("eventdriven_Rejected")
        eventdriven_rejected.Activities.Add(m_BugRejectedEvent)

        ' Listen to see if it was worked.
        Dim listen_bug_worked As New ListenActivity("listen_Worked")
        listen_bug_worked.Activities.Add(eventdriven_worked)
        listen_bug_worked.Activities.Add(eventdriven_rejected)

        ' Test the bug fix.
        m_TestBug = New CallExternalMethodActivity("invoke_TestBug")
        m_TestBug.InterfaceType = GetType(IBugReportService)
        m_TestBug.MethodName = "TestBug"

        ' If the bug was fixed, test it.
        eventdriven_worked.Activities.Add(m_TestBug)

        ' Listen for either the TestPassed or TestFailed event.
        ' Catch the TestPassed event.
        m_TestPassedEvent = New HandleExternalEventActivity("catch_TestPassed")
        m_TestPassedEvent.InterfaceType = GetType(IBugReportService)
        m_TestPassedEvent.EventName = "TestPassed"

        ' Catch the TestFailed event.
        m_TestFailedEvent = New HandleExternalEventActivity("catch_TestFailed")
        m_TestFailedEvent.InterfaceType = GetType(IBugReportService)
        m_TestFailedEvent.EventName = "TestFailed"

        ' Make activities driven by the BugWorked and BugRejected events.
        Dim eventdriven_passed As New EventDrivenActivity("eventdriven_Passed")
        eventdriven_passed.Activities.Add(m_TestPassedEvent)
        Dim eventdriven_failed As New EventDrivenActivity("eventdriven_Failed")
        eventdriven_failed.Activities.Add(m_TestFailedEvent)

        ' Listen to see if it was worked.
        Dim listen_test_passed As New ListenActivity("listen_Passed")
        listen_test_passed.Activities.Add(eventdriven_passed)
        listen_test_passed.Activities.Add(eventdriven_failed)

        ' After we test, listen for these events.
        eventdriven_worked.Activities.Add(listen_test_passed)

        ' Add the activities.
        Me.Activities.Add(m_AssignEngineer)
        Me.Activities.Add(m_EngineerAssignedEvent)
        Me.Activities.Add(m_WorkBug)
        Me.Activities.Add(listen_bug_worked)

        Me.CanModifyActivities = False
    End Sub

    ' Save the assigned employee's name.
    Private Sub m_EngineerAssignedEvent_Invoked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Handles m_EngineerAssignedEvent.Invoked
        ' Convert to AssignedEngineerArgs.
        Dim assigned_args As StringArgs = DirectCast(e, StringArgs)

        ' Save the value.
        m_AssignedEngineer = assigned_args.Value

        Debug.WriteLine("Assigned: " & m_AssignedEngineer)
    End Sub

    ' We are about to invoke this method.
    ' Copy the bug's description into the activity's parameter.
    Private Sub m_AssignEngineer_MethodInvoking(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_AssignEngineer.MethodInvoking
        m_AssignEngineer.ParameterBindings.Item("bug_description").Value = m_BugDescription
    End Sub
    Private Sub m_WorkBug_MethodInvoking(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_WorkBug.MethodInvoking
        m_WorkBug.ParameterBindings.Item("bug_description").Value = m_BugDescription
    End Sub
    Private Sub m_TestBug_MethodInvoking(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_TestBug.MethodInvoking
        m_TestBug.ParameterBindings.Item("bug_description").Value = m_BugDescription
    End Sub

    ' The bug was rejected.
    Private Sub m_BugRejectedEvent_Invoked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Handles m_BugRejectedEvent.Invoked
        Debug.WriteLine("Bug rejected")
        Debug.WriteLine("")
        Status = "Rejected"
    End Sub

    ' The bug was worked.
    Private Sub m_BugWorkedEvent_Invoked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Handles m_BugWorkedEvent.Invoked
        Debug.WriteLine("Bug worked")
    End Sub

    ' The test failed.
    Private Sub m_TestFailedEvent_Invoked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Handles m_TestFailedEvent.Invoked
        Debug.WriteLine("Test failed")
        Debug.WriteLine("")
        Status = "Test Failed"
    End Sub

    ' The test passed.
    Private Sub m_TestPassedEvent_Invoked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Handles m_TestPassedEvent.Invoked
        Debug.WriteLine("Test passed")
        Debug.WriteLine("")
        Status = "Test Passed"
    End Sub
End Class

<Serializable()> _
Public Class StringArgs
    Inherits ExternalDataEventArgs

    Public Value As String

    Public Sub New(ByVal instance_id As Guid, ByVal new_value As String)
        MyBase.New(instance_id)
        Value = new_value
    End Sub
End Class